@@ -2,6 +2,8 @@
 Tools for writing a nexus file
 """
 
+import collections
+
 TEMPLATE = """
 #NEXUS
 %(comments)s
@@ -14,7 +16,7 @@ MATRIX
 ;
 END;
 """
-from .reader import NexusReader
+
 
 class NexusWriter:
 
@@ -23,44 +25,44 @@ class NexusWriter:
     DATATYPE = 'STANDARD'
 
     def __init__(self):
-        self.taxalist = []
         self.comments = []
-        self.characters = []
-        self.clean_characters = {}
-        self.symbols = set()
-        self.data = {}
+        self.data = collections.defaultdict(dict)
         self.is_binary = False
 
     def clean(self, s):
         """Removes unsafe characters"""
+        _EMPTY = ''
         replacements = {
-            ' ': '', '\\': '', '(': '_', ')': '', ':': '',
-            '/': '', '?': '', '-': '',
+            ' ': _EMPTY, '\\': _EMPTY, ':': _EMPTY,
+            '/': _EMPTY, '?': _EMPTY, '-': _EMPTY,
+            '(': '_', ')': _EMPTY,
         }
         for f, t in replacements.items():
             s = s.replace(f, t)
         return s
-
-    def _add_char(self, charlabel):
-        """Adds a character"""
-        charlabel = str(charlabel)
-        if charlabel not in self.characters:
-            self.characters.append(charlabel)
-            self.data[charlabel] = {}
-            self.clean_characters[charlabel] = self.clean(charlabel)
-        return charlabel
-
-    def _add_taxa(self, taxon):
-        """Adds a taxa"""
-        if taxon not in self.taxalist:
-            self.taxalist.append(taxon)
-        return taxon
-
+    
+    @property
+    def characters(self):
+        return self.data.keys()
+    
+    @property
+    def taxalist(self):
+        t = set()
+        [t.update(self.data[c].keys()) for c in self.data]
+        return t
+    
+    @property
+    def symbols(self):
+        symbols = set()
+        [symbols.update(self.data[c].values()) for c in self.data]
+        symbols = [s for s in symbols if s not in ('-', '?')]
+        return symbols
+    
     def _make_charlabel_block(self):
         """Generates a character label block"""
         out = ["CHARSTATELABELS"]
-        for i, c in enumerate(sorted(self.characters), 1):
-            out.append("\t\t%d %s," % (i, self.clean_characters[c]))
+        for i, char in enumerate(sorted(self.characters), 1):
+            out.append("\t\t%d %s," % (i, self.clean(str(char))))
         out[-1] = out[-1].strip(',')  # remove trailing comma
         out.append(";")
         return "\n".join(out)
@@ -96,13 +98,11 @@ class NexusWriter:
     def add_comment(self, comment):
         """Adds a `comment` into the nexus file"""
         self.comments.append(comment)
-
+    
     def add(self, taxon, character, value):
         """Adds a `character` for the given `taxon` and sets it to `value`"""
         assert self.is_binary is False, \
             "Unable to add data to a binarised nexus form"
-        character = self._add_char(character)
-        taxon = self._add_taxa(taxon)
         value = str(value)
         # have multiple entries
         if taxon in self.data[character]:
@@ -110,10 +110,6 @@ class NexusWriter:
         else:
             self.data[character][taxon] = value
 
-        # add to symbols
-        if value not in ['?', '-']:
-            [self.symbols.add(v) for v in value]
-
     def write(self, interleave=False, charblock=False):
         """
         Generates a string representation of the nexus
@@ -198,6 +194,7 @@ class NexusWriter:
         One day I'll refactor this all so Reader and Writer subclass something,
         which will make this unnecessary.
         """
+        from .reader import NexusReader
         n = NexusReader()
         out = self.make_nexus(interleave=False, charblock=True)
         n.read_string(out)
